#include <windows.h>
#include <commctrl.h>
#include <stdio.h>

#include "ScriptInterpreter.h"
#include "ScriptError.h"
#include "ScriptValue.h"

#include "resource.h"
#include "filter.h"

///////////////////////////////////////////////////////////////////////////

int pixelclutterRunProc(const FilterActivation *fa, const FilterFunctions *ff);
int pixelclutterStartProc(FilterActivation *fa, const FilterFunctions *ff);
int pixelclutterEndProc(FilterActivation *fa, const FilterFunctions *ff);
long pixelclutterParamProc(FilterActivation *fa, const FilterFunctions *ff);
int pixelclutterConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd);
void pixelclutterStringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str);
void pixelclutterScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc);
bool pixelclutterFssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen);

///////////////////////////////////////////////////////////////////////////
HWND hwndTrack;

typedef struct MyFilterData {
 	int		deger;
} MyFilterData;

ScriptFunctionDef pixelclutter_func_defs[]={
    { (ScriptFunctionPtr)pixelclutterScriptConfig, "Config", "0ii" },
    { NULL },
};

CScriptObject pixelclutter_obj={
    NULL, pixelclutter_func_defs
};

struct FilterDefinition filterDef_pixelclutter = {

    NULL, NULL, NULL,				// next, prev, module
    "Pixel Clutter",				// name
    "Pixelleri Clutter Filter..",
									// desc
    "Engin KARTAL",					// maker
    NULL,							// private_data
    sizeof(MyFilterData),			// inst_data_size

    NULL,							// initProc
    NULL,							// deinitProc
    pixelclutterRunProc,			// runProc
    pixelclutterParamProc,			// paramProc
    pixelclutterConfigProc,			// configProc
    pixelclutterStringProc,			// stringProc
    pixelclutterStartProc,			// startProc
    pixelclutterEndProc,			// endProc

    &pixelclutter_obj,				// script_obj
    pixelclutterFssProc,			// fssProc

};

///////////////////////////////////////////////////////////////////////////

extern "C" int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat);
extern "C" void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff);

static FilterDefinition *fd_pixelclutter;

///////////////////////////////////////////////////////////////////////////

int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat) {
    if (!(fd_pixelclutter = ff->addFilter(fm, &filterDef_pixelclutter, sizeof(FilterDefinition))))
        return 1;

	vdfd_ver = VIRTUALDUB_FILTERDEF_VERSION;
	vdfd_compat = VIRTUALDUB_FILTERDEF_COMPATIBLE;

    return 0;
}

void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff) {
    ff->removeFilter(fd_pixelclutter);
}

///////////////////////////////////////////////////////////////////////////

static bool g_MMXenabled;



HWND  CreateTrackbar( 
    HWND hwndDlg,  // handle of dialog box (parent window) 
    UINT iMin,     // minimum value in trackbar range 
    UINT iMax,     // maximum value in trackbar range 
    UINT iSelMin,  // minimum value in trackbar selection 
    UINT iSelMax)  // maximum value in trackbar selection 
{ 

  //  InitCommonControls(); // loads common control's DLL 

    hwndTrack = CreateWindowEx( 
        0,                             // no extended styles 
        TRACKBAR_CLASS,                // class name 
        "Trackbar Control",            // title (caption) 
        WS_CHILD | WS_VISIBLE | 
        TBS_AUTOTICKS | TBS_ENABLESELRANGE,  // style 
        7, 7,                        // position 
        250, 30,                       // size 
        hwndDlg,                       // parent window 
        NULL,             // control identifier 
        0,                       // instance 
        NULL                           // no WM_CREATE parameter 
        ); 

    SendMessage(hwndTrack, TBM_SETRANGE, 
        (WPARAM) TRUE,                   // redraw flag 
        (LPARAM) MAKELONG(iMin, iMax));  // min. & max. positions 
    SendMessage(hwndTrack, TBM_SETPAGESIZE, 
        0, (LPARAM) 4);                  // new page size 

    SendMessage(hwndTrack, TBM_SETSEL, 
        (WPARAM) FALSE,                  // redraw flag 
        (LPARAM) MAKELONG(iSelMin, iSelMax)); 
    SendMessage(hwndTrack, TBM_SETPOS, 
        (WPARAM) TRUE,                   // redraw flag 
        (LPARAM) iSelMin); 

    SetFocus(hwndTrack); 

    return hwndTrack; 
} 



int pixelclutterStartProc(FilterActivation *fa, const FilterFunctions *ff) {

    return 0;
}


int pixelclutterRunProc(const FilterActivation *fa, const FilterFunctions *ff) {

    MyFilterData *mfd = (MyFilterData *)fa->filter_data;
    PixDim w, h;
	int i,j,n,k,yi,yj;
	Pixel32 *old_pixel;
	
    h = fa->src.h;
	w = fa->src.w;
	n = mfd->deger;

	for (j=0;j<h;j++)
		for (i=0;i<w;i++){
		
			old_pixel = fa->src.Address32(i,j);
		//	*old_pixel = *old_pixel & 0xFF0000;
			
			k = ((n/2)-n) + rand() % n;
			yi = i + k;
			k = ((n/2)-n) + rand() % n;
			yj = j + k;

			if (yi<0) yi	= 0;
			if (yj<0) yj	= 0;
			if (yi>=w) yi	= w - 1;
			if (yj>=h) yj	= h - 1;

			*(fa->dst.Address(yi,yj)) = *old_pixel;

		}
    return 0;
}


int pixelclutterEndProc(FilterActivation *fa, const FilterFunctions *ff) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;


    return 0;
}

long pixelclutterParamProc(FilterActivation *fa, const FilterFunctions *ff) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;


	fa->dst.pitch = (fa->dst.w*4 + 7) & -8;

    return FILTERPARAM_SWAP_BUFFERS;
}


BOOL CALLBACK pixelclutterConfigDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam) {
    MyFilterData *mfd = (MyFilterData *)GetWindowLong(hdlg, DWL_USER);

    switch(msg) {
        case WM_INITDIALOG:
            SetWindowLong(hdlg, DWL_USER, lParam);
            mfd = (MyFilterData *)lParam;
			CreateTrackbar(hdlg,1,10,0,0);
         	SendMessage(hwndTrack, TBM_SETPOS, (WPARAM) TRUE, (LPARAM) (mfd->deger));
            return TRUE;

        case WM_COMMAND:
            switch(LOWORD(wParam)) {
            case IDOK:
         		mfd->deger =  SendMessage(hwndTrack, TBM_GETPOS, 0, 0); 
                EndDialog(hdlg, 0);
                return TRUE;
            case IDCANCEL:
                EndDialog(hdlg, 1);
                return FALSE;
            }
            break;
    }

    return FALSE;
}

int pixelclutterConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd) {
    return DialogBoxParam(fa->filter->module->hInstModule,
            MAKEINTRESOURCE(IDD_FILTER_DIALOG), hwnd,
            pixelclutterConfigDlgProc, (LPARAM)fa->filter_data);
}

void pixelclutterStringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	
    _snprintf(str, 10, " - %d",
        mfd->deger);
}

void pixelclutterScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc) {
    FilterActivation *fa = (FilterActivation *)lpVoid;
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	mfd->deger		= argv[0].asInt();
}

bool pixelclutterFssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

    _snprintf(buf, buflen, "Config(%d)",
        mfd->deger);

    return true;
}

